import os
import shutil
import traceback

from framework import F, logger
from support import SupportYaml, d


class MenuManager:
    menu_map = None

    @classmethod
    def __load_menu_yaml(cls):
        try:
            menu_yaml_filepath = os.path.join(F.config['path_data'], 'db', 'menu.yaml')
            if os.path.exists(menu_yaml_filepath) == False:
                shutil.copy(
                    os.path.join(F.config['path_app'], 'files', 'menu.yaml.template'),
                    menu_yaml_filepath
                )
            cls.menu_map = SupportYaml.read_yaml(menu_yaml_filepath)
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())
            cls.menu_map = SupportYaml.read_yaml(os.path.join(F.config['path_app'], 'files', 'menu.yaml.template'))
        

    @classmethod
    def init_menu(cls):
        cls.__load_menu_yaml()
        if cls.__init_menu() == False:
            cls.menu_map = SupportYaml.read_yaml(os.path.join(F.config['path_app'], 'files', 'menu.yaml.template'))
            cls.__init_menu()

    @classmethod
    def __init_menu(cls):
        try:
            from .init_plugin import PluginManager
            plugin_menus = PluginManager.plugin_menus
            copy_map = [] 
            for category in cls.menu_map:
                if 'uri' in category:
                    if category['uri'] in plugin_menus:
                        plugin_menus[category['uri']]['match'] = True
                        copy_map.append(plugin_menus[category['uri']]['menu'])
                    else:
                        copy_map.append(category)
                    continue
                cate_count = 0  

                tmp_cate_list = []
                for item in category['list']:
                    if item['uri'] in plugin_menus:
                        plugin_menus[item['uri']]['match'] = True
                        tmp_cate_list.append(plugin_menus[item['uri']]['menu'])
                        cate_count += 1
                    elif item['uri'].startswith('http'):
                        tmp_cate_list.append({
                            'uri': item['uri'],
                            'name': item['name'],
                            'target': item.get('target', '_blank')
                        })
                        cate_count += 1
                    elif (len(item['uri'].split('/')) > 1 and item['uri'].split('/')[0] in plugin_menus) or item['uri'].startswith('javascript') or item['uri'] in ['-']:
                        tmp_cate_list.append({
                            'uri': item['uri'],
                            'name': item.get('name', ''),
                        })
                        cate_count += 1
                    elif item['uri'] == 'setting':
                        # 2024.06.04
                        # 확장설정도 메뉴 구성
                        if len(PluginManager.setting_menus) > 0:
                            set_tmp = item.get('list')
                            if set_tmp:
                                cp = PluginManager.setting_menus.copy()
                                include = [] 
                                for set_ch in set_tmp:
                                    if set_ch.get('uri') and (set_ch.get('uri') == '-' or set_ch.get('uri').startswith('http')):
                                        include.append(set_ch)
                                        continue

                                    for i, ps in enumerate(cp):
                                        if set_ch.get('plugin') != None and set_ch.get('plugin') == ps.get('plugin'):
                                            include.append(ps)
                                            del cp[i]
                                            break
                                tmp_cate_list.append({
                                    'uri': item['uri'],
                                    'name': item.get('name', ''),
                                    'list': include + cp
                                })
                           
                            else:
                                tmp_cate_list.append({
                                    'uri': item['uri'],
                                    'name': item.get('name', ''),
                                    'list': PluginManager.setting_menus
                                })
                    
                if cate_count > 0:
                    copy_map.append({
                        'name': category['name'],
                        'list': tmp_cate_list,
                        'count': cate_count
                    })
            cls.menu_map = copy_map
                
            make_dummy_cate = False
            for name, plugin_menu in plugin_menus.items():
                if plugin_menu['match'] == False:
                    if make_dummy_cate == False:
                        make_dummy_cate = True
                        cls.menu_map.insert(len(cls.menu_map)-1, {
                            'name':'미분류', 'count':0, 'list':[]
                        })
                    c = cls.menu_map[-2]
                    c['count'] += 1
                    c['list'].append(plugin_menu['menu'])
            return True
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())
            return False


    @classmethod
    def get_menu_map(cls):
        return cls.menu_map


    @classmethod
    def get_setting_menu(cls, plugin):
        from .init_plugin import PluginManager
        for tmp in PluginManager.setting_menus:
            if tmp['plugin'] == plugin:
                return tmp
    


#f = open('/data/PIPY/flaskfarm/flaskfarm/lib/framework/version.py').read()
import platform

import setuptools

#with open("README.md", "r") as fh:
#    long_description = fh.read()


if platform.system() == 'Windows':
    f = open('C:\\work\\FlaskFarm\\flaskfarm_support\\pipy\\flaskfarm\\flaskfarm\\lib\\framework\\version.py').read()
else:
    f = open('/data/flaskfarm_support/pipy/flaskfarm/flaskfarm/lib/framework/version.py').read()

version = f.split('=')[1].replace('"', '').strip()
print(version)

setuptools.setup(
    name="FlaskFarm", # Replace with your own username
    version=version,
    author="flaskfarm",
    author_email="flaskfarm@gmail.com",
    description="FlaskFarm",
    long_description="",
    long_description_content_type="text/markdown",
    url="https://github.com/flaskfarm/flaskfarm",
    #packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.9',

    #package_dir={"flaskfarm": "src"},
    #packages=setuptools.find_packages(where="src"),
    #packages=['flaskfarm'],
    #py_modules=['flaskfarm'],
    #package_data={'flaskfarm' :['flaskfarm/files/*','flaskfarm/lib/*']}

    packages = [
        'flaskfarm',
        'flaskfarm.files',
        'flaskfarm.cli',
        'flaskfarm.lib.framework',
        'flaskfarm.lib.framework.static.css',
        'flaskfarm.lib.framework.static.css.theme',
        'flaskfarm.lib.framework.static.js',
        'flaskfarm.lib.framework.static.file',
        'flaskfarm.lib.framework.static.img',
        'flaskfarm.lib.framework.templates',
        'flaskfarm.lib.plugin',
        'flaskfarm.lib.support',
        'flaskfarm.lib.support.base',
        'flaskfarm.lib.support.expand',
        'flaskfarm.lib.support.libsc',
        'flaskfarm.lib.system',
        'flaskfarm.lib.system.templates',
        'flaskfarm.lib.tool',
    ],

    package_data={
        'flaskfarm.files':['*'],
        'flaskfarm.cli':['*'],
        'flaskfarm.lib.framework':['*'],
        'flaskfarm.lib.framework.static.css':['*'],
        'flaskfarm.lib.framework.static.css.theme':['*'],
        'flaskfarm.lib.framework.static.js':['*'],
        'flaskfarm.lib.framework.static.file':['*'],
        'flaskfarm.lib.framework.static.img':['*'],
        'flaskfarm.lib.framework.templates':['*'],
        'flaskfarm.lib.plugin':['*'],
        'flaskfarm.lib.support':['*'],
        'flaskfarm.lib.support.base':['*'],
        'flaskfarm.lib.support.expand':['*'],
        'flaskfarm.lib.support.libsc':['*'],
        'flaskfarm.lib.system':['*'],
        'flaskfarm.lib.system.templates':['*'],
        'flaskfarm.lib.tool':['*'],
    },

    install_requires=[
        "Flask",
        "Flask-SQLAlchemy",
        "Flask-Login",
        "Flask-Cors",
        #"Flask-Markdown",
        "Flask-SocketIO",
        "python-engineio",
        "python-socketio",
        "Werkzeug",
        "Jinja2",
        "markupsafe",
        "itsdangerous",
        "apscheduler",
        "pytz",
        "requests",
        "discord-webhook",
        "pyyaml",
        "telepot-mod",
        "Flask-Dropzone",
        "psutil",
        "pillow",
        "gevent",
        "gevent-websocket",
        "pycryptodome",
        #"celery",
        #"redis",
        "json_fix",
    ],
)



import logging
import logging.handlers
import os
import sys
from datetime import datetime

from pytz import timezone, utc

"""
ConsoleColor.Black => "\x1B[30m",
            ConsoleColor.DarkRed => "\x1B[31m",
            ConsoleColor.DarkGreen => "\x1B[32m",
            ConsoleColor.DarkYellow => "\x1B[33m",
            ConsoleColor.DarkBlue => "\x1B[34m",
            ConsoleColor.DarkMagenta => "\x1B[35m",
            ConsoleColor.DarkCyan => "\x1B[36m",
            ConsoleColor.Gray => "\x1B[37m",
            ConsoleColor.Red => "\x1B[1m\x1B[31m",
            ConsoleColor.Green => "\x1B[1m\x1B[32m",
            ConsoleColor.Yellow => "\x1B[1m\x1B[33m",
            ConsoleColor.Blue => "\x1B[1m\x1B[34m",
            ConsoleColor.Magenta => "\x1B[1m\x1B[35m",
            ConsoleColor.Cyan => "\x1B[1m\x1B[36m",
            ConsoleColor.White => "\x1B[1m\x1B[37m",
"""

class CustomFormatter(logging.Formatter):
    """Logging Formatter to add colors and count warning / errors"""

    grey = "\x1b[38;21m"
    yellow = "\x1b[33;21m"
    red = "\x1b[31;21m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    green = "\x1B[32m"
    # pathname filename
    #format = "[%(asctime)s|%(name)s|%(levelname)s - %(message)s (%(filename)s:%(lineno)d)"
    
    __format = '[{yellow}%(asctime)s{reset}|{color}%(levelname)s{reset}|{green}%(name)s{reset}|%(pathname)s:%(lineno)s] {color}%(message)s{reset}' if os.environ.get('LOGGER_PATHNAME', "False") == "True" else '[{yellow}%(asctime)s{reset}|{color}%(levelname)s{reset}|{green}%(name)s{reset}|%(filename)s:%(lineno)s] {color}%(message)s{reset}'

    FORMATS = {
        logging.DEBUG: __format.format(color=grey, reset=reset, yellow=yellow, green=green),
        logging.INFO: __format.format(color=green, reset=reset, yellow=yellow, green=green),
        logging.WARNING: __format.format(color=yellow, reset=reset, yellow=yellow, green=green),
        logging.ERROR: __format.format(color=red, reset=reset, yellow=yellow, green=green),
        logging.CRITICAL: __format.format(color=bold_red, reset=reset, yellow=yellow, green=green)
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def get_logger(name=None, log_path=None):
    if name == None:
        name = sys.argv[0].rsplit('.', 1)[0]
    logger = logging.getLogger(name)
    if not logger.handlers:
        level = logging.DEBUG
        logger.setLevel(level)
        formatter = logging.Formatter(u'[%(asctime)s|%(levelname)s|%(name)s|%(filename)s:%(lineno)s] %(message)s')
        def customTime(*args):
            utc_dt = utc.localize(datetime.utcnow())
            my_tz = timezone("Asia/Seoul")
            converted = utc_dt.astimezone(my_tz)
            return converted.timetuple()

        formatter.converter = customTime
        file_max_bytes = 1 * 1024 * 1024 
        if log_path == None:
            log_path = os.path.join(os.getcwd(), 'tmp')
        os.makedirs(log_path, exist_ok=True)
        fileHandler = logging.handlers.RotatingFileHandler(filename=os.path.join(log_path, f'{name}.log'), maxBytes=file_max_bytes, backupCount=5, encoding='utf8', delay=True)
        streamHandler = logging.StreamHandler() 

        fileHandler.setFormatter(formatter)
        streamHandler.setFormatter(CustomFormatter()) 
        
        logger.addHandler(fileHandler)
        logger.addHandler(streamHandler)
    return logger




import re
import traceback

from . import logger


class SupportString(object):
    @classmethod
    def get_cate_char_by_first(cls, title):  # get_first
        value = ord(title[0].upper())
        if ord('가') <= value < ord('나'): return '가'
        if ord('나') <= value < ord('다'): return '나'
        if ord('다') <= value < ord('라'): return '다'
        if ord('라') <= value < ord('마'): return '라'
        if ord('마') <= value < ord('바'): return '마'
        if ord('바') <= value < ord('사'): return '바'
        if ord('사') <= value < ord('아'): return '사'
        if ord('아') <= value < ord('자'): return '아'
        if ord('자') <= value < ord('차'): return '자'
        if ord('차') <= value < ord('카'): return '차'
        if ord('카') <= value < ord('타'): return '카'
        if ord('타') <= value < ord('파'): return '타'
        if ord('파') <= value < ord('하'): return '파'
        if ord('하') <= value < ord('힣'): return '하'
        return '0Z'
    

    @classmethod
    def is_include_hangul(cls, text):
        try:
            hanCount = len(re.findall(u'[\u3130-\u318F\uAC00-\uD7A3]+', text))
            return hanCount > 0
        except:
            return False

    @classmethod
    def language_info(cls, text):
        try:
            text = text.strip().replace(' ', '')
            all_count = len(text)
            han_count = len(re.findall('[\u3130-\u318F\uAC00-\uD7A3]', text))
            eng_count = len(re.findall('[a-zA-Z]', text))
            etc_count = len(re.findall('[0-9]', text))
            etc_count += len(re.findall('[-=+,#/\?:^$.@*\"※~&%ㆍ!』\\‘|\(\)\[\]\<\>`\'…》：]', text))
            if all_count == etc_count:
                return (0,0)
            han_percent = int(han_count * 100 / (all_count-etc_count))
            eng_percent = int(eng_count * 100 / (all_count-etc_count))
            return (han_percent, eng_percent)
        except Exception as e: 
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())
            return False

    @classmethod
    def remove_special_char(cls, text):
        return re.sub('[-=+,#/\?:^$.@*\"※~&%ㆍ!』\\‘|\(\)\[\]\<\>`\'…》：]', '', text)


    @classmethod
    def remove_emoji(cls, text, char=''):
        import re
        emoji_pattern = re.compile("["
                                u"\U0001F600-\U0001F64F"  # emoticons
                                u"\U0001F300-\U0001F5FF"  # symbols & pictographs
                                u"\U0001F680-\U0001F6FF"  # transport & map symbols
                                u"\U0001F1E0-\U0001F1FF"  # flags (iOS)
                                u"\U00002500-\U00002BEF"  # chinese char
                                u"\U00002702-\U000027B0"
                                u"\U00002702-\U000027B0"
                                #u"\U000024C2-\U0001F251"
                                u"\U0001f926-\U0001f937"
                                u"\U00010000-\U0010ffff"
                                u"\u2640-\u2642"
                                u"\u2600-\u2B55"
                                u"\u200d"
                                u"\u23cf"
                                u"\u23e9"
                                u"\u231a"
                                u"\ufe0f"  # dingbats
                                u"\u3030"
                                "]+", flags=re.UNICODE)
        # Remove emojis from the text
        text = emoji_pattern.sub(char, text)
        return text


import argparse
import os
import platform
import re
import sys
import traceback

from . import logger


class SupportSC:
    LIBRARY_LOADING = False
    try:
        sys.path.append(os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'libsc'))
        import sc
        LIBRARY_LOADING = True
    except:
        pass
    

    @classmethod
    def encode(cls, text, mode=0):
        try:
            import sc
            return sc.encode(text, mode)
        except Exception as e:
            logger.error(f'Exception:{str(e)}')
            logger.error(traceback.format_exc())
            return None

        
    @classmethod
    def decode(cls, text):
        try:
            import sc
            return sc.decode(text)
        except:
            return None
    
    @classmethod
    def load_module(cls, module_name, module_code):
        try:
            import sc
            mod = sc.load_module(module_name, module_code)
            sys.modules[mod] = mod
            logger.warning(f"C-LOADING : {module_name}")
            return mod
        except Exception as e:
            logger.error(f'Exception:{str(e)}')
            logger.error(traceback.format_exc())

    @classmethod
    def load_module_P(cls, plugin_ins, filename):
        return cls.load_module_f(plugin_ins.setting['filepath'], filename)

    @classmethod
    def load_module_f(cls, package_filepath, filename):
        dirname = os.path.dirname(package_filepath)
        filepath = os.path.join(dirname, filename + '.pyf')
        if os.path.exists(filepath) and os.path.exists(filepath):
            from support import SupportFile
            code = SupportFile.read_file(filepath)
            return cls.load_module(f"{os.path.basename(dirname)}.{filename}", code)

    @classmethod
    def td(self, mediacode, ts, url):
        try:
            import sc
            ret = sc.td1(mediacode, str(ts), url).strip()
            ret = re.sub('[^ -~]+', '', ret).strip()
            return ret
        except:
            return None



import os
import traceback

try:
    from slack_sdk.webhook import WebhookClient
except:
    os.system('pip3 install slack-sdk')
    from slack_sdk.webhook import WebhookClient
from . import logger

class SupportSlack:
    @classmethod
    def send_slack_message(cls, text, webhook_url=None, image_url=None, disable_notification=None):
        try:
            if webhook_url is None:
                return False
            webhook = WebhookClient(webhook_url)
            if image_url is not None:
                webhook.send(text=text, blocks=[{"type": "image", "title": {"type": "plain_text", "text": "Image", "emoji": True}, "image_url": image_url, "alt_text": "Image"}])
            webhook.send(text=text)
            return True
        except Exception as e:
            logger.error(f"Exception:{str(e)}")
            logger.error(traceback.format_exc())
        return False